From 21acdecdeec65bc1f847c58d5afffd390e826c73 Mon Sep 17 00:00:00 2001
From: Thomas Munro <thomas.munro@gmail.com>
Date: Wed, 14 Aug 2024 09:19:13 +1200
Subject: [PATCH] Harmonize MinGW CODESET lookup with MSVC.

Historically, MinGW environments lacked some Win32 calls, so we took a
different code path in win32_langinfo().  Somehow, the code change in
commit 35eeea62 (removing setlocale() calls) caused one particular
001_initdb.pl test to fail on MinGW + ICU builds, because
pg_import_system_collations() found no collations.  It might take a
MinGW user to discover the exact reason.

Updating that function to use the same code as MSVC seems to fix that
test, so lets do that.  (There are plenty more places that test for MSVC
unnecessarily, to be investigated later.)

While here, also rename the helper function win32_langinfo() to
win32_get_codeset(), to explain what it does less confusingly.
---
 src/port/chklocale.c | 9 +++------
 1 file changed, 3 insertions(+), 6 deletions(-)

diff --git a/src/port/chklocale.c b/src/port/chklocale.c
index 99e27ed6de..9506cd87ed 100644
--- a/src/port/chklocale.c
+++ b/src/port/chklocale.c
@@ -193,7 +193,7 @@ static const struct encoding_match encoding_match_list[] = {
 
 #ifdef WIN32
 /*
- * On Windows, use CP<code page number> instead of the nl_langinfo() result
+ * On Windows, use CP<code page number> instead of CODESET.
  *
  * This routine uses GetLocaleInfoEx() to parse short locale names like
  * "de-DE", "fr-FR", etc.  If those cannot be parsed correctly process falls
@@ -203,12 +203,10 @@ static const struct encoding_match encoding_match_list[] = {
  * Returns a malloc()'d string for the caller to free.
  */
 static char *
-win32_langinfo(const char *ctype)
+win32_get_codeset(const char *ctype)
 {
 	char	   *r = NULL;
 	char	   *codepage;
-
-#if defined(_MSC_VER)
 	uint32		cp;
 	WCHAR		wctype[LOCALE_NAME_MAX_LENGTH];
 
@@ -233,7 +231,6 @@ win32_langinfo(const char *ctype)
 		}
 	}
 	else
-#endif
 	{
 		/*
 		 * Locale format on Win32 is <Language>_<Country>.<CodePage>.  For
@@ -336,7 +333,7 @@ pg_get_encoding_from_locale(const char *ctype, bool write_message)
 
 	freelocale(loc);
 #else
-	sys = win32_langinfo(ctype);
+	sys = win32_get_codeset(ctype);
 #endif
 
 	if (!sys)
-- 
2.39.2

